#!/usr/bin/perl
# $Id: mymassagecomments.pl,v 1.1 2002/01/17 03:11:13 scottu Exp $

use File::Find;
use Getopt::Long;

my $mapfile = undef;        # author map file

$result = GetOptions (
	'map=s'      => \$mapfile,
);

exit 1 if (!$result);

# load author map if we have one
my %authormap;
if ($mapfile)
{
	open (MAP, "<$mapfile") || die;

	for $mapline (<MAP>)
	{
		if ($mapline =~ /(\S+)\s+(\S+)/) { $authormap{$1} = $2; }
	}
	close (MAP);
}

#input record separator;
$irs = $/;
#ouput record separator;
$ors = $\;

sub massage_file
{
	$thefile = $_;

	# ignore directories, link files, and non ,v files
	return if (-d $thefile);
	return if ($thefile =~ /\.lnk\.sh,v$/);
	return if ($thefile !~ /,v$/);

	print "Massaging \"$thefile\" ... ";

	open (COMMAVFILE, "<$thefile");

	binmode COMMAVFILE;

	my @linebuffer;
	my %revdateline;
	my $inrevs = false;
	my @filedump;

	# read all of the "version/date/branches/next" lines, removing the date
	# lines:
	#
	# 1.2
	# date	2000.10.08.17.26.41;	author laine;	state Exp;
	# branches;
	# next	1.1;
	#

	# The last of these lines are followed by a line that says only "desc"

	while ($fileline = <COMMAVFILE>) {
		# save the lines to regurgitate later
		push @linebuffer,$fileline;
		if ($fileline =~ /^desc$/) {last;}
		# parse out revision number
		if ($fileline =~ /^([\d]+(\.[\d]+)+)$/) {$rev = $1;}
		elsif ($fileline =~ /^date\s[\d]{2,4}(\.[\d]{2}){5};.*/) {
			# remove the original date line, but keep it in the "dateline
			# array" in case there isn't one in the comments
			$revdateline{$rev} = pop @linebuffer;
		}
	}

	# Now go through all the "rev/log/comment/text" lines, looking for
	# date lines in the comments. When one is found, replace the original
	# date line in the array

	while ($fileline = <COMMAVFILE>) {
		push @linebuffer,$fileline;
		if ($fileline =~ /^([\d]+(\.[\d]+)+)$/) {$rev = $1;}
		elsif ($fileline =~ /^date\s[\d]{2,4}(\.[\d]{2}){5};.*/) {

			# tsu -if we have an author map, use it to change the author now
			if ($mapfile && $fileline =~ /author ([^;]+);/)
			{
				$orig = $1;
				$new = $authormap{$orig};

				if (defined ($new))
				{
					$fileline =~ s/author $orig;/author $new;/;
				}
			}

			$revdateline{$rev} = $fileline;
			pop @linebuffer;
		}
	}

	close COMMAVFILE;

	open (TEMPFILE, "> tempfile.$$");

	binmode TEMPFILE;

	# print out all of the "version/date/branches/next" lines, re-adding
	# the date line right after the version line.

	while ($fileline = shift @linebuffer) {
		print TEMPFILE $fileline;
		if ($fileline =~ /^([\d]+(\.[\d]+)+)$/) {
			$rev = $1;
			print TEMPFILE $revdateline{$rev};
		}
		if ($fileline =~ /^desc$/) {last;}
	}

	# print all the "rev/log/comment/text" lines.

	while ($fileline = shift @linebuffer) {
		print TEMPFILE $fileline;
	}

	close TEMPFILE;

	# move the new file in place of the old one.
	system("rm -f \"$thefile\"");
	system("mv -f tempfile.$$ \"$thefile\"");
	print "OK\n";
}

@ARGV = qw(.) unless @ARGV;

find (\&massage_file, @ARGV);
